#!/usr/bin/env node
const path = require("path");
const fs = require("fs");
const utils = require("../dist/utils");

const yargs = require("yargs").option("config", {
	alias: "c",
	describe: "Path to the config file",
	type: "string",
	default: "ezpack.config.js"
});

const argv = yargs.argv;
const configPath = path.resolve(argv.config);

/**
 * 判断配置文件是否存在
 */
if (!fs.existsSync(configPath)) {
	utils.printError(`config path: ${configPath} not find.`);
	console.log();
	yargs.showHelp();
	return;
}

const cmd = argv._[0];
const config = require(configPath);
const { EZPack } = require("../dist/EZPack");

const ezpack = new EZPack(config);
switch (cmd) {
	case "start":
		ezpack.start();
		break;
	case "build":
		ezpack.build().catch(e => {
			utils.printError(e);
		});
		break;
	case "publish":
		ezpack.publish();
		break;
	case "analyse":
		ezpack.build(true);
		break;
	default:
		yargs.showHelp();
}
