"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const date_fns_1 = require("date-fns");
const fse = require("fs-extra");
exports.HOME_DIR = ".ezpack";
class Cache {
    constructor(filename) {
        this.readFile = () => {
            return fse.readFileSync(this.cacheFile, "utf8");
        };
        this.writeToFile = () => {
            return fse.writeFile(this.cacheFile, JSON.stringify(this.cacheInfo), "utf8");
        };
        this.cacheFile = `./${exports.HOME_DIR}/${filename}`;
        fse.ensureFileSync(this.cacheFile);
    }
}
exports.Cache = Cache;
class LastBuiltEntryCache extends Cache {
    constructor() {
        super("last-build-entry-cache.json");
        try {
            this.cacheInfo = JSON.parse(this.readFile());
        }
        catch (_a) {
            this.cacheInfo = [];
        }
    }
    getEntry() {
        return this.cacheInfo;
    }
    refreshEntry(newEntry) {
        this.cacheInfo = newEntry;
        this.writeToFile();
    }
}
exports.LastBuiltEntryCache = LastBuiltEntryCache;
class DevEntryCache extends Cache {
    constructor() {
        super("dev-entry-cache.json");
        try {
            this.cacheInfo = JSON.parse(this.readFile());
            const { createTime } = this.cacheInfo;
            if (createTime && createTime < this.currentTime) {
                this.cacheInfo = {
                    entryList: [],
                    createTime: this.currentTime
                };
            }
        }
        catch (_a) {
            this.cacheInfo = {
                entryList: [],
                createTime: this.currentTime
            };
        }
        this.currentTime = date_fns_1.getTime(date_fns_1.startOfToday());
    }
    getEntry() {
        return this.cacheInfo.entryList;
    }
    addEntry(entryName) {
        const entryList = this.cacheInfo.entryList;
        if (!entryList.includes(entryName)) {
            entryList.push(entryName);
            this.writeToFile();
        }
    }
}
exports.DevEntryCache = DevEntryCache;
exports.lastBuiltEntryCache = new LastBuiltEntryCache();
exports.devEntryCache = new DevEntryCache();
//# sourceMappingURL=CacheData.js.map