"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const webpack = require("webpack");
const ora = require("ora");
const chalk = require("chalk");
const webpackConfig_1 = require("./webpackConfig");
const startServer_1 = require("./startServer");
const entry_1 = require("./utils/entry");
const inquirer = require("inquirer");
const _ = require("lodash");
const utils_1 = require("./utils");
const path = require("path");
const fse = require("fs-extra");
const vfs = require("vinyl-fs");
const Git_1 = require("./utils/Git");
function validateConfig(config) {
    return config;
}
exports.validateConfig = validateConfig;
function selectEnv(envList) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log();
        envList.forEach((env, index) => {
            console.log(`${index + 1}: ${env}`);
        });
        console.log();
        let answers = yield inquirer.prompt([
            {
                type: "input",
                name: "num",
                message: "Please input the env number"
            }
        ]);
        if (!_.isFinite(+answers.num)) {
            utils_1.printError("Invalid env num.");
            process.exit(1);
        }
        const env = envList[answers.num - 1];
        if (env) {
            return env;
        }
        else {
            utils_1.printError(`Env ${answers.num} not found.`);
            process.exit(1);
            return "";
        }
    });
}
class EZPack {
    constructor(options) {
        this.options = options;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const { webpack: webpackConfig, name, devServer } = this.options;
            const config = webpackConfig_1.genDevConfig(webpackConfig, name, devServer.port);
            startServer_1.default(config, devServer);
        });
    }
    build(analyse) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentRepo = new Git_1.default();
            const currentBranch = yield currentRepo.getCurrentBranch();
            const isOnline = currentBranch === "master";
            const { webpack: webpackConfig, disableEntrySelect } = this.options;
            let { entry } = webpackConfig;
            const entryLen = Object.keys(entry).length;
            if (entryLen > 1) {
                entry = yield (disableEntrySelect === true
                    ? entry_1.getEntryByKey(["all"], webpackConfig.entry, webpackConfig.rootPath)
                    : entry_1.readCustomEntry(webpackConfig.entry, webpackConfig.rootPath, true));
                if (!entry) {
                    utils_1.printWarn("No entry found.");
                    return;
                }
            }
            const config = webpackConfig_1.genProdConfig(Object.assign({}, webpackConfig, { entry }), this.options.name, isOnline, analyse);
            const spinner = ora("webpack building...");
            spinner.start();
            return new Promise((resolve, reject) => {
                webpack(config, (err, stats) => {
                    spinner.stop();
                    if (err) {
                        reject(err);
                    }
                    else if (stats.hasErrors()) {
                        reject(stats.toJson().errors.join("\n"));
                    }
                    else {
                        process.stdout.write(stats.toString({
                            colors: true,
                            modules: false,
                            children: false,
                            chunks: false,
                            chunkModules: false
                        }) + "\n\n");
                        console.log(chalk.cyan("  Build complete.\n"));
                        // if (entryLen > 1) {
                        // 	// only update auto-entry when having multiple entries
                        // 	updateAutoEntry();
                        // }
                        utils_1.alertSuccess("Build successfully.");
                        resolve();
                    }
                });
            });
        });
    }
    publish() {
        return __awaiter(this, void 0, void 0, function* () {
            const { publish: publishOptions, webpack: webpackOptions, name: projectName } = this.options;
            const currentRepo = new Git_1.default();
            const currentBranch = yield currentRepo.getCurrentBranch();
            const isOnline = currentBranch === "master";
            let publishPath = "";
            let mapPath = "";
            let env = "";
            if (isOnline) {
                const answers = yield inquirer.prompt([
                    {
                        type: "confirm",
                        name: "isPublish",
                        default: false,
                        message: "Are you sure to publish online?"
                    }
                ]);
                if (!answers.isPublish) {
                    return;
                }
                env = "online";
                const { mapDir, onlinePath } = publishOptions;
                publishPath = onlinePath;
                if (mapDir) {
                    mapPath = path.join(publishPath, mapDir, "map.js");
                }
            }
            else {
                const { uatPath, uatEnv, uatMapName, mapDir } = publishOptions;
                if (uatEnv && uatEnv.length > 0) {
                    env = yield selectEnv(uatEnv);
                    publishPath = path.resolve(uatPath, env);
                }
                else {
                    publishPath = uatPath;
                    if (uatMapName) {
                        env = yield selectEnv(uatMapName);
                        mapPath = path.join(uatPath, mapDir, env);
                    }
                }
            }
            if (!(yield utils_1.isDirectory(publishPath))) {
                utils_1.printError(`publish fail! ${publishPath} not exists.`);
                return;
            }
            const publishRepo = new Git_1.default(publishPath);
            // make sure publishPath is a clean git repository
            try {
                const stats = yield publishRepo.status();
                if (stats.files.length !== 0) {
                    utils_1.printError(`${publishPath} is not clean, check it.`);
                    return;
                }
            }
            catch (e) {
                utils_1.printError(`${publishPath} is not a git repository.`);
                return;
            }
            publishRepo.pull("origin", "master");
            try {
                yield fse.emptyDir(webpackOptions.outputPath);
                yield this.build();
            }
            catch (e) {
                utils_1.printError(e);
                return;
            }
            if (mapPath) {
                yield fse.move(path.join(webpackOptions.outputPath, "map.js"), mapPath, {
                    overwrite: true
                });
            }
            vfs.src(path.join(webpackOptions.outputPath, "**/*")).pipe(vfs.dest(publishPath));
            const answers = yield inquirer.prompt([
                {
                    type: "confirm",
                    name: "isUpload",
                    default: false,
                    message: "Auto publish:"
                }
            ]);
            if (answers.isUpload) {
                try {
                    yield publishRepo.add("./*");
                    yield publishRepo.commit(`Auto publish ${projectName} ${currentBranch}.`);
                    yield publishRepo.pull("origin", "master", {
                        "--rebase": "true"
                    });
                    yield publishRepo.push("master");
                    utils_1.printSuccess(`Publish ${env} successfully.`);
                }
                catch (e) {
                    utils_1.printWarn("Build successfully, but auto publish fail.");
                    return;
                }
            }
        });
    }
}
exports.EZPack = EZPack;
//# sourceMappingURL=EZPack.js.map