"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const git = require("simple-git/promise");
const path = require("path");
class Git {
    constructor(gitPath) {
        this.git = git(gitPath);
    }
    status() {
        return this.git.status();
    }
    pull(remote, branch, options) {
        return this.git.pull(remote, branch, options);
    }
    push(branch, remote = "origin") {
        return __awaiter(this, void 0, void 0, function* () {
            if (!branch) {
                branch = yield this.getCurrentBranch();
            }
            return this.git.push(remote, branch);
        });
    }
    add(p) {
        return this.git.add(p);
    }
    commit(msg) {
        return this.git.commit(msg);
    }
    getLatestCommit() {
        return this.git.log(["-1"]).then(log => log.latest.hash);
    }
    getFilesByCommitRange(from, to) {
        return __awaiter(this, void 0, void 0, function* () {
            const commits = yield this.git
                .log({
                from,
                to
            })
                .then(log => log.all.map(item => item.hash));
            return this.getFilesByCommits(commits);
        });
    }
    getFilesByCommits(commits) {
        return this.git.show(["--name-only", "--pretty=", ...commits]).then((result) => result
            .split("\n")
            .filter(item => !!item)
            .map(file => path.resolve(process.cwd(), file)));
    }
    getCurrentBranch() {
        return __awaiter(this, void 0, void 0, function* () {
            const stats = yield this.git.status();
            return stats.current;
        });
    }
}
exports.default = Git;
//# sourceMappingURL=Git.js.map