"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ora = require("ora");
const fse = require("fs-extra");
const inquirer = require("inquirer");
const SearchCheckbox = require("inquirer-search-checkbox");
const Git_1 = require("./Git");
const parse_1 = require("./parse");
const index_1 = require("./index");
const CacheData_1 = require("../CacheData");
inquirer.registerPrompt("SearchCheckbox", SearchCheckbox);
const CACHE_FILE = `./${CacheData_1.HOME_DIR}/auto-entry`;
const git = new Git_1.default();
const ALL_KEY = "<ALL>";
const AUTO_KEY = "auto";
const LAST_KEY = "<LAST>";
function updateAutoEntry() {
    return __awaiter(this, void 0, void 0, function* () {
        const latestCommit = yield git.getLatestCommit();
        const files = yield git.getFilesByCommits([latestCommit]);
        if (files.length === 1 && files[0].includes("auto-entry")) {
            // prevent useless cycle commit
            return;
        }
        fse.writeFile(CACHE_FILE, latestCommit, "utf8").then(() => __awaiter(this, void 0, void 0, function* () {
            const msg = "Auto update ezpack auto-entry";
            yield git.add(CACHE_FILE);
            yield git.commit(msg);
            git.push().catch(e => {
                index_1.printWarn(`${msg} fail.`);
            });
        }));
    });
}
exports.updateAutoEntry = updateAutoEntry;
function getAutoEntry() {
    fse.ensureFileSync(CACHE_FILE);
    return fse.readFileSync(CACHE_FILE, "utf8").replace("\n", "");
}
function autoSelectEntry(entry, rootPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const allPackageMap = {};
        const entryKeyList = Object.keys(entry);
        const spinner = ora("Finding entries...");
        spinner.start();
        return Promise.all(Object.keys(entry).map(k => {
            const item = entry[k];
            let entryFile = item;
            if (Array.isArray(item)) {
                entryFile = item.reverse()[0];
            }
            allPackageMap[k] = {};
            return parse_1.getPackages(allPackageMap[k], entryFile, rootPath);
        })).then((packages) => __awaiter(this, void 0, void 0, function* () {
            const saveCommit = getAutoEntry();
            const latestCommit = yield git.getLatestCommit();
            spinner.stop();
            if (saveCommit && saveCommit !== latestCommit) {
                const files = yield git.getFilesByCommitRange(saveCommit, latestCommit);
                const changeKeyList = entryKeyList.filter(k => files.some(file => allPackageMap[k][file]));
                if (changeKeyList.length > 0) {
                    console.log("Below entries will be built: ");
                    changeKeyList.forEach(k => {
                        console.log("    - " + k);
                    });
                    console.log();
                    return changeKeyList.reduce((obj, k) => {
                        obj[k] = entry[k];
                        return obj;
                    }, {});
                }
                return null;
            }
            return null;
        }));
    });
}
function getEntryByKey(key, entry, rootPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (key.includes(ALL_KEY)) {
            return Promise.resolve(entry);
        }
        else if (key.includes(AUTO_KEY)) {
            return yield autoSelectEntry(entry, rootPath);
        }
        else if (key.includes(LAST_KEY)) {
            const lastEntry = CacheData_1.lastBuiltEntryCache.getEntry();
            if (lastEntry.length === 0) {
                return Promise.resolve(null);
            }
            else {
                key = CacheData_1.lastBuiltEntryCache.getEntry();
                index_1.printInfo(`Building entry: ${key.join(", ")}`);
            }
        }
        CacheData_1.lastBuiltEntryCache.refreshEntry(key);
        const outputEntry = key.reduce(function (pValue, cValue) {
            if (typeof entry[cValue] !== "undefined") {
                pValue[cValue] = entry[cValue];
            }
            return pValue;
        }, {});
        return Promise.resolve(outputEntry);
    });
}
exports.getEntryByKey = getEntryByKey;
function readCustomEntry(entry, rootPath, auto) {
    return __awaiter(this, void 0, void 0, function* () {
        if (Array.isArray(entry)) {
            // 数组entry直接返回;
            return entry;
        }
        let keys = [ALL_KEY, LAST_KEY].concat(Object.keys(entry));
        const answers = yield inquirer.prompt([
            {
                type: "SearchCheckbox",
                name: "key",
                message: `Select your webpack entry (${keys.length})`,
                choices: keys.map(name => ({
                    name
                })),
                pageSize: 6,
                validate: function (answer) {
                    if (answer.length < 1) {
                        return "You must choose at least one entry.";
                    }
                    return true;
                }
            }
        ]);
        return getEntryByKey(answers.key, entry, rootPath);
    });
}
exports.readCustomEntry = readCustomEntry;
//# sourceMappingURL=entry.js.map