"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const fse = require("fs-extra");
const os = require("os");
const notifier = require("node-notifier");
exports.printInfo = msg => {
    console.log();
    console.log(chalk.blue(msg));
};
exports.printError = msg => {
    console.log();
    console.log(chalk.red(msg));
};
exports.printSuccess = msg => {
    console.log();
    console.log(chalk.green(msg));
};
exports.printWarn = msg => {
    console.log();
    console.log(chalk.yellow(msg));
};
function isExist(pathName, directory) {
    return fse
        .stat(pathName)
        .then(stats => {
        if (stats) {
            if (directory) {
                return stats.isDirectory();
            }
            else {
                return stats.isFile();
            }
        }
        return false;
    })
        .catch(() => false);
}
function isDirectory(dirPathName) {
    return isExist(dirPathName, true);
}
exports.isDirectory = isDirectory;
function getLocalIP() {
    const ifaces = os.networkInterfaces();
    const ipList = [];
    Object.keys(ifaces).forEach(ifname => {
        let alias = 0;
        ifaces[ifname].forEach(iface => {
            if (iface.family !== "IPv4" || iface.internal !== false) {
                // skip over internal (i.e. 127.0.0.1) and non-ipv4 addresses
                return;
            }
            if (alias >= 1) {
                // this single interface has multiple ipv4 addresses
                console.log(ifname + ":" + alias, iface.address);
            }
            else {
                // this interface has only one ipv4 adress
                ipList.push(iface.address);
            }
            ++alias;
        });
    });
    let ip = "";
    if (ipList.length > 0) {
        ip = ipList.find(item => item.includes("192"));
    }
    if (ip) {
        return ip;
    }
    else {
        exports.printWarn("Can not get your local ip, using 127.0.0.1");
        return "127.0.0.1";
    }
}
exports.localIP = getLocalIP();
function alertSuccess(message) {
    notifier.notify({
        title: "ezpack",
        message
    });
}
exports.alertSuccess = alertSuccess;
//# sourceMappingURL=index.js.map