"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const babylon = require("babylon");
const path = require("path");
const t = require("babel-types");
const babel_core_1 = require("babel-core");
const fse = require("fs-extra");
function getPackageName(node) {
    return t.isTemplateLiteral(node.arguments[0])
        ? node.arguments[0].quasis[0].value.raw
        : node.arguments[0].value;
}
const PARSE_PLUGINS = [
    "jsx",
    "typescript",
    "asyncFunctions",
    "classConstructorCall",
    "doExpressions",
    "trailingFunctionCommas",
    "objectRestSpread",
    "decorators",
    "classProperties",
    "exportExtensions",
    "exponentiationOperator",
    "asyncGenerators",
    "functionBind",
    "functionSent"
];
const ext = [".js", ".jsx", ".ts", ".tsx"];
function resolveEXT(pathName) {
    const extList = ["/index.js", "/index.jsx", "/index.ts", "/index.tsx", ...ext];
    return extList.map(ext => pathName + ext).find(p => {
        try {
            const stat = fse.statSync(p);
            return !!stat;
        }
        catch (e) {
            return false;
        }
    });
}
function getPackages(packageMap, filePath, rootPath) {
    return __awaiter(this, void 0, void 0, function* () {
        return fse.readFile(filePath, "utf8").then(content => {
            let packages = [];
            const visitor = {
                ImportDeclaration(path) {
                    packages.push(path.node.source.value);
                },
                CallExpression(path) {
                    if (path.node.callee.name === "require") {
                        packages.push(getPackageName(path.node));
                    }
                }
            };
            const ast = babylon.parse(content, {
                sourceType: "module",
                plugins: PARSE_PLUGINS
            });
            babel_core_1.traverse(ast, visitor);
            const fileDirPath = filePath
                .split("/")
                .reverse()
                .slice(1)
                .reverse()
                .join("/");
            packages = packages
                .filter(name => name.indexOf("/") !== -1)
                .map(name => {
                if (name.startsWith("./") || name.startsWith("../")) {
                    return path.join(fileDirPath, name);
                }
                else {
                    return path.join(rootPath, name);
                }
            })
                .map(resolveEXT)
                .filter(p => p && !packageMap[p]);
            packages.forEach(name => {
                packageMap[name] = true;
            });
            if (packages.length > 0) {
                return Promise.all(packages
                    .filter(name => ext.includes(path.extname(name)))
                    .map(name => getPackages(packageMap, name, rootPath)));
            }
            return Promise.resolve([]);
        });
    });
}
exports.getPackages = getPackages;
//# sourceMappingURL=parse.js.map