const TsTypeInfo = require("ts-type-info");
const path = require("path");
const fs = require("fs");
const ejs = require("ejs");

const name = path.resolve(__dirname, "../src/EZPack.ts");
const result = TsTypeInfo.getInfoFromFiles([name])
	.getFile(name)
	.getInterface("EZPackOptions");

const tables = [];

function visit(node) {
	if (!node.properties) {
		return;
	}

	const t = {};
	t.name = node.name;
	t.properties = [];

	const dependency = [];
	node.properties.forEach(p => {
		if (p.documentationComment) {
			t.properties.push({
				name: p.name,
				type: p.type.text,
				doc: p.documentationComment
					.replace(/\*/g, "")
					.replace("/", "")
					.replace("/", "")
					.trim()
			});

			const definitions = p.type.definitions;

			if (definitions.length === 1) {
				const d = definitions[0];
				if (d.name !== "Entry") {
					dependency.push(d);
				}
			} else if (p.type.definitions.length > 1) {
				console.log("Unexpected definition length");
			}
		}
	});

	if (t.properties.length > 0) {
		tables.push(t);
		if (dependency.length) {
			dependency.forEach(d => visit(d));
		}
	}
}

visit(result);

const tplFile = path.resolve(__dirname, "./doc.ejs");
const htmlPath = path.resolve(__dirname, "../doc.html");
ejs.renderFile(tplFile, { options: tables }, {}, function(err, str) {
	if (err) {
		console.log(err);
	} else {
		fs.writeFile(htmlPath, str, function name(err) {
			if (err) {
				return console.log(err);
			}

			console.log("Generate document successfully!");
		});
	}
});
